@@ -34,16 +34,39 @@
 #include <sys/times.h>
 #include <gtk/gtk.h>
 #include <errno.h>
+#include <locale.h>
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#ifdef HAVE_OPENMP
+#include <omp.h>
+#endif
+
+#include "measurements.h"
+
+// Define GSourceOnceFunc if compiling against an older version of GLIB:
+#if GLIB_VERSION_CUR_STABLE < G_ENCODE_VERSION(2,74)
+typedef void (* GSourceOnceFunc) (gpointer user_data);
+#endif
+
 #undef _Complex_I
 #define _Complex_I  (__extension__ 1.0i)
 #undef I
 #define I _Complex_I
 
+/* GNU/Hurd has no hard PATH_MAX */
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
+// Use __LOCATION__ anywhere you want the file and line number:
+// (Move this to be used in all calls to mem_realloc)
+#define __S1(x) #x
+#define __S2(x) __S1(x)
+#define  __LOCATION__ __FILE__ ":"  __S2(__LINE__)
+
 /*
  * Standard gettext macros.
  */
@@ -230,6 +253,9 @@ typedef struct Segment
 
 #define CHILD   isChild()
 
+/* Maximum length for mathlib ID strings (including null terminator) */
+#define MATHLIB_ID_LEN 32
+
 /* Runtime configuration data */
 typedef struct
 {
@@ -240,7 +266,7 @@ typedef struct
   char input_file[FILENAME_LEN];
 
   /* If set true, then use GTK loops for the frequency loop iteration
-   * instead of spawing a pthread */
+   * instead of spawning a pthread */
   int disable_pthread_freqloop;
 
   /* Main (structure) window position and size */
@@ -268,6 +294,7 @@ typedef struct
 
   /* Radiation pattern window position and size */
   int
+    rdpattern_is_open,
     rdpattern_x,
     rdpattern_y,
     rdpattern_width,
@@ -284,6 +311,7 @@ typedef struct
 
   /* Frequency plots window state of widgetsposition and size */
   int
+    freqplots_is_open,
     freqplots_x,
     freqplots_y,
     freqplots_width,
@@ -313,8 +341,9 @@ typedef struct
   /* Enable Quit dialog */
   int confirm_quit;
 
-  /* Preferred mathlib index, if available */
-  int mathlib_idx, mathlib_batch_idx;
+  /* Preferred mathlib identifiers for persistence */
+  char mathlib_id[MATHLIB_ID_LEN];
+  char mathlib_batch_id[MATHLIB_ID_LEN];
 
   /* See enum GAIN_SCALE */
   int gain_style;
@@ -323,8 +352,38 @@ typedef struct
    * fr_plot->min_fscale/fr_plot->max_fscale values. */
   int freqplots_round_x_axis;
 
+  /* if true, exit after the first frequency loop iteration */
+  int batch_mode;
+
+  /* verbose and debug levels, see console.h */
+  int verbose, debug;
+
+  /* true if ~/.xnec2c/xnec2c.conf does not exist, false otherwise */
+  int first_run;
+
+  /* optimization file write flags */
+  int opt_write_csv;
+  int opt_write_s1p;
+  int opt_write_s2p_max_gain;
+  int opt_write_s2p_viewer_gain;
+  int opt_write_rdpat;
+  int opt_write_currents;
+
+  /* If for example --write-csv <file> is added to the cmdline then write
+   * the file after each frequency iteration */
+  char *filename_csv;
+  char *filename_s1p;
+  char *filename_s2p_max_gain;
+  char *filename_s2p_viewer_gain;
+  char *filename_rdpat;
+  char *filename_currents;
 } rc_config_t;
 
+typedef struct {
+	void (*callback)(char *);
+	char *extension;
+} filechooser_t;
+
 /* Gain Scaling style */
 enum GAIN_SCALE
 {
@@ -335,6 +394,9 @@ enum GAIN_SCALE
   NUM_SCALES
 };
 
+/* Minimum gain value used for color mapping */
+#define COLOR_MIN_GAIN -60.0
+
 /* Polarization type */
 enum POL_TYPE
 {
@@ -518,7 +580,8 @@ typedef struct
     ksymp,  /* Ground flag */
     ifar,   /* Int flag in RP card, for far field calculations */
     iperf,  /* Type of ground flag */
-    nradl;  /* Number of radials in ground screen */
+    nradl,  /* Number of radials in ground screen */
+    gpflag; /* The gpflag from the GE card */
 
   double
     t2,     /* Const for radial wire ground impedance */
@@ -831,8 +894,7 @@ typedef struct
     *max_gain_tht,  /* Theta angle where maximum gain occurs */
     *max_gain_phi,  /*   Phi angle where maximum gain occurs */
     *tilt,          /* Tilt angle of polarization ellipse  */
-    *axrt,          /* Elliptic axial ratio of pol ellipse */
-    fbratio;        /* Front to Back gain ratio */
+    *axrt;          /* Elliptic axial ratio of pol ellipse */
 
   int
     *max_gain_idx,  /* Where in rad_pattern.gtot the max value occurs */
@@ -1289,6 +1351,7 @@ void Cairo_Draw_Line(cairo_t *cr, int x1, int y1, int x2, int y2);
 void Cairo_Draw_Lines(cairo_t *cr, GdkPoint *points, int npoints);
 /* draw_radiation.c */
 int Draw_Radiation(cairo_t *cr);
+gboolean Validate_Nearfield_Animation(void);
 gboolean Animate_Near_Field(gpointer udata);
 double Polarization_Factor(int pol_type, int fstep, int idx);
 void Set_Polarization(int pol);
@@ -1327,7 +1390,7 @@ void unere(double xob, double yob, double zob);
 /* fork.c */
 void Child_Process(int num_child);
 ssize_t Write_Pipe(int idx, char *str, ssize_t len, gboolean err);
-void Get_Freq_Data(int idx, int fstep);
+int Get_Freq_Data(int idx, int fstep);
 /* geom_edit.c */
 void Wire_Editor(int action);
 void Patch_Editor(int action);
@@ -1352,9 +1415,15 @@ gboolean patch(int nx, int ny, double ax1, double ay1, double az1, double ax2, d
 gboolean reflc(int ix, int iy, int iz, int iti, int nop);
 void wire(double xw1, double yw1, double zw1, double xw2, double yw2, double zw2, double rad, double rdel, double rrad, int ns, int itg);
 /* gnuplot.c */
+void Save_FreqPlots_S1P(char *filename);
+void Save_FreqPlots_S2P_Max_Gain(char *filename);
+void Save_FreqPlots_S2P_Viewer_Gain(char *filename);
+void Save_FreqPlots_CSV(char *filename);
 void Save_FreqPlots_Gnuplot_Data(char *filename);
 void Save_RadPattern_Gnuplot_Data(char *filename);
+void Save_RadPattern_CSV(char *filename);
 void Save_Struct_Gnuplot_Data(char *filename);
+void Save_Currents_CSV(char *filename);
 /* ground.c */
 void rom2(double a, double b, _Complex double *sum, double dmin);
 void sflds(double t, _Complex double *e);
@@ -1414,6 +1483,7 @@ int solve(int n, _Complex double *a, int *ip, _Complex double *b, int ndim);
 int solve_gauss_elim( int n, complex double *a, int *ip, complex double *b, int ndim );
 void solves(_Complex double *a, int *ip, _Complex double *b, int neq, int nrh, int np, int n, int mp, int m);
 /* nec2_model.c */
+void Zero_Store(GtkListStore *store, GtkTreeIter *iter, int ncols, int start_idx, int stop_idx);
 void Nec2_Input_File_Treeview(int action);
 void cell_edited_callback(GtkCellRendererText *cell, gchar *path, gchar *new_text, gpointer user_data);
 void Save_Nec2_Input_File(GtkWidget *treeview_window, char *nec2_file);
@@ -1423,11 +1493,12 @@ void load(int *ldtyp, int *ldtag, int *ldtagf, int *ldtagt, double *zlr, double
 /* optimize.c */
 void Write_Optimizer_Data(void);
 void *Optimizer_Output(void *arg);
+int opt_have_files_to_save(void);
 /* plot_freqdata.c */
 void Plot_Frequency_Data(cairo_t *cr);
 void Plots_Window_Killed(void);
 void Set_Frequency_On_Click(GdkEvent *event);
-int freqplots_click_pending();
+int freqplots_click_pending(void);
 /* radiation.c */
 void rdpat(void);
 /* rc_config.c */
@@ -1446,9 +1517,13 @@ int Stop(char *mesg, int err);
 int Notice(char *title, char *message,  GtkButtonsType buttons);
 gboolean Nec2_Save_Warn(const gchar *mesg);
 int Load_Line(char *buff, FILE *pfile);
+void xnec2c_set_omp_cpus(void);
+void clock_print_elapsed(char *msg, clockid_t clk_id, struct timespec *start);
+void clock_print_elapsed_when(char *msg, clockid_t clk_id, struct timespec *start, float min_sec);
 void mem_alloc(void **ptr, size_t req, gchar *str);
 void mem_realloc(void **ptr, size_t req, gchar *str);
 void mem_backtrace(void *ptr);
+void mem_obj_dump(void *ptr);
 void free_ptr(void **ptr);
 gboolean Open_File(FILE **fp, char *fname, const char *mode);
 void Close_File(FILE **fp);
@@ -1462,10 +1537,13 @@ void SaveFlag(unsigned long long int *flag, unsigned long long int mask);
 void Strlcpy(char *dest, const char *src, size_t n);
 void Strlcat(char *dest, const char *src, size_t n);
 double Strtod(char *nptr, char **endptr);
+char *str_append(char *dst, char *a, char *b, size_t n);
 void Get_Dirname(char *fpath, char *dirname, int *fname_idx);
 void xnec2_widget_queue_draw(GtkWidget *w);
-void g_idle_add_once(GSourceFunc function, gpointer data);
-void g_idle_add_once_sync(GSourceFunc function, gpointer data);
+#if GLIB_VERSION_CUR_STABLE < G_ENCODE_VERSION(2,74)
+guint g_idle_add_once(GSourceOnceFunc function, gpointer data);
+#endif
+guint g_idle_add_once_sync(GSourceOnceFunc function, gpointer data);
 void print_backtrace(char *msg);
 
 /* xnec2c.c */
@@ -1477,5 +1555,6 @@ gboolean Frequency_Loop(gpointer udata);
 gboolean Start_Frequency_Loop(void);
 void Stop_Frequency_Loop(void);
 void Incident_Field_Loop(void);
+int set_freq_step(void);
 
 #endif
