@@ -20,9 +20,24 @@
 #include "draw_radiation.h"
 #include "shared.h"
 
+/* Constants for display layout */
+#define TEXT_GRADIENT_SPACING 8  /* Spacing between text and gradient bar in pixels */
+#define LINE_WIDTH 2            /* Width of lines in pixels */
+
+static const char *nearfield_animation_error_msg =
+  N_("Animation requires near field data.\n\n"
+     "E-field animation: Add NE card to NEC file\n"
+     "H-field animation: Add NH card to NEC file\n"
+     "Poynting vector: Add both NE and NH cards");
+
 /* For coloring rad pattern */
 static double *red = NULL, *grn = NULL, *blu = NULL;
 
+/* Function prototypes */
+void Draw_Color_Legend_Overlay( cairo_t *cr );
+
+int need_rdpat_redraw = 1;
+
 /* Buffered points in 3d (xyz) space
  * forming the radiation pattern */
 static point_3d_t *point_3d = NULL;
@@ -101,7 +116,7 @@ Draw_Radiation_Pattern( cairo_t *cr )
   double dph = (double)fpat.dph * (double)TORAD;
 
   /* Used to set text in labels */
-  gchar txt[8];
+  gchar txt[16];
 
   /* Abort if rad pattern cannot be drawn */
   fstep = calc_data.freq_step;
@@ -128,9 +143,13 @@ Draw_Radiation_Pattern( cairo_t *cr )
     rdpattern_proj_params.r_max =
       Scale_Gain( rad_pattern[fstep].gtot[idx], fstep, idx);
 
-    /* Distance of rdpattern point nearest to xyz origin */
+    /* Get actual minimum gain for calculations and display */
     idx = rad_pattern[fstep].min_gain_idx[pol];
-    r_min = Scale_Gain( rad_pattern[fstep].gtot[idx], fstep, idx);
+    double actual_gain = rad_pattern[fstep].gtot[idx];
+    
+    /* For color mapping, use COLOR_MIN_GAIN as the floor */
+    double color_gain = (actual_gain < COLOR_MIN_GAIN) ? COLOR_MIN_GAIN : actual_gain;
+    r_min = Scale_Gain(color_gain, fstep, idx);
 
     /* Range of scaled rdpattern gain values */
     r_range = rdpattern_proj_params.r_max - r_min;
@@ -189,8 +208,8 @@ Draw_Radiation_Pattern( cairo_t *cr )
       {
         Value_to_Color(
             &red[col_idx], &grn[col_idx], &blu[col_idx],
-            (point_3d[pts_idx].r+point_3d[pts_idx+1].r)/2.0-r_min,
-            r_range );
+            ((point_3d[pts_idx].r+point_3d[pts_idx+1].r)/2.0-r_min)/r_range,
+            1.0);
         col_idx++;
         pts_idx++;
 
@@ -221,13 +240,15 @@ Draw_Radiation_Pattern( cairo_t *cr )
     } /* for( nph = 1; nph < fpat.nph; nph++ ) */
 
     /* Show max gain on color code bar */
-    snprintf( txt, 8, "%6.1f", rad_pattern[fstep].max_gain[pol] );
+    snprintf( txt, sizeof(txt)-1, "%.2f", rad_pattern[fstep].max_gain[pol] );
     gtk_label_set_text( GTK_LABEL(Builder_Get_Object(
             rdpattern_window_builder, "rdpattern_colorcode_maxlabel")),
         txt );
 
-    /* Show min gain on color code bar */
-    snprintf( txt, 6, "%4.1f", rad_pattern[fstep].min_gain[pol] );
+    /* Show min gain on color code bar, clamped to COLOR_MIN_GAIN */
+    double color_min_gain = rad_pattern[fstep].min_gain[pol];
+    if (color_min_gain < COLOR_MIN_GAIN) color_min_gain = COLOR_MIN_GAIN;
+    snprintf( txt, sizeof(txt)-1, "%.2f", color_min_gain );
     gtk_label_set_text(GTK_LABEL(Builder_Get_Object(
             rdpattern_window_builder, "rdpattern_colorcode_minlabel")),
         txt );
@@ -318,6 +339,9 @@ Draw_Radiation_Pattern( cairo_t *cr )
     } /* for( nph = 1; nph < fpat.nph; nph++ ) */
   } /* for( nth = 0; nth < fpat.nth; nth++ ) */
 
+  /* Draw color legend overlay */
+  Draw_Color_Legend_Overlay( cr );
+
   /* Show gain in direction of viewer */
   Show_Viewer_Gain(
       rdpattern_window_builder,
@@ -552,6 +576,9 @@ Draw_Near_Field( cairo_t *cr )
     return;
   }
 
+  /* Draw color legend overlay */
+  Draw_Color_Legend_Overlay( cr );
+
   /* Show max field strength on color code bar */
   if( isFlagSet(DRAW_EFIELD) )
     max = near_field.max_er;
@@ -589,13 +616,8 @@ _Draw_Radiation( cairo_t *cr )
       (double)rdpattern_proj_params.height);
   cairo_fill( cr );
 
-  /* Abort if xnec2c may be quit by user */
-  if( isFlagSet(MAIN_QUIT) || isFlagClear(ENABLE_EXCITN) )
-    return FALSE;
-
-  /* Don't draw radiation pattern when freq
-   * loop is running and optimizer enabled */
-  if( isFlagSet(OPTIMIZER_OUTPUT) && isFlagSet(FREQ_LOOP_RUNNING) )
+  /* Abort if excitation (EX card) is missing */
+  if( isFlagClear(ENABLE_EXCITN) )
     return FALSE;
 
   // Try to hold the lock to prevent drawing the radiation pattern
@@ -623,6 +645,11 @@ int Draw_Radiation( cairo_t *cr )
 {
 	int ret;
 
+	if (isFlagSet(ERROR_CONDX))
+		return FALSE;
+
+	need_rdpat_redraw = 0;
+
 	g_mutex_lock(&freq_data_lock);
 	ret = _Draw_Radiation( cr );
 	g_mutex_unlock(&freq_data_lock);
@@ -630,6 +657,34 @@ int Draw_Radiation( cairo_t *cr )
 	return ret;
 }
 
+/*-----------------------------------------------------------------------*/
+
+gboolean
+Validate_Nearfield_Animation( void )
+{
+  if( ((isFlagSet(DRAW_EFIELD) || isFlagSet(DRAW_POYNTING)) && !(fpat.nfeh & NEAR_EFIELD)) ||
+      ((isFlagSet(DRAW_HFIELD) || isFlagSet(DRAW_POYNTING)) && !(fpat.nfeh & NEAR_HFIELD)) )
+  {
+    Notice( _("Near Field Animation"),
+        _(nearfield_animation_error_msg),
+        GTK_BUTTONS_OK );
+    return( FALSE );
+  }
+
+  if( ((isFlagSet(DRAW_EFIELD) || isFlagSet(DRAW_POYNTING)) &&
+       (near_field.fex == NULL || near_field.fey == NULL || near_field.fez == NULL)) ||
+      ((isFlagSet(DRAW_HFIELD) || isFlagSet(DRAW_POYNTING)) &&
+       (near_field.fhx == NULL || near_field.fhy == NULL || near_field.fhz == NULL)) )
+  {
+    Notice( _("Near Field Animation"),
+        _(nearfield_animation_error_msg),
+        GTK_BUTTONS_OK );
+    return( FALSE );
+  }
+
+  return( TRUE );
+}
+
 /*-----------------------------------------------------------------------*/
 
   gboolean
@@ -642,6 +697,19 @@ Animate_Near_Field( gpointer udata )
   if( isFlagClear(NEAREH_ANIMATE) )
     return( FALSE );
 
+  if( !Validate_Nearfield_Animation() )
+  {
+    ClearFlag( NEAREH_ANIMATE );
+    if( anim_tag > 0 )
+    {
+      g_source_remove( anim_tag );
+      anim_tag = 0;
+    }
+    if( animate_dialog != NULL )
+      Gtk_Widget_Destroy( &animate_dialog );
+    return( FALSE );
+  }
+
   /* Number of points in near fields */
   npts = fpat.nrx * fpat.nry * fpat.nrz;
   for( idx = 0; idx < npts; idx++ )
@@ -838,9 +906,10 @@ New_Radiation_Projection_Angle(void)
   rdpattern_proj_params.sin_wi = sin(rdpattern_proj_params.Wi/(double)TODEG);
   rdpattern_proj_params.cos_wi = cos(rdpattern_proj_params.Wi/(double)TODEG);
 
+  need_rdpat_redraw = 1;
+
   /* Trigger a redraw of radiation drawingarea */
-  if( isFlagSet(DRAW_ENABLED) && 
-	  (isFlagClear(OPTIMIZER_OUTPUT) || isFlagClear(FREQ_LOOP_RUNNING)))
+  if( isFlagSet(DRAW_ENABLED) )
   {
     xnec2_widget_queue_draw( rdpattern_drawingarea );
   }
@@ -864,10 +933,10 @@ New_Radiation_Projection_Angle(void)
   gboolean
 Redo_Radiation_Pattern( gpointer udata )
 {
-  /* Redo radiation pattern for a new frequency. Below
-   * makes calcs use the extra buffer in rad_pattern FIXME */
-  calc_data.freq_step = calc_data.steps_total;
-  New_Frequency();
+  // Only re-calculate if the set_freq_step() determines that
+  // the selected frequency has not been calculated.
+  if (!set_freq_step())
+	  New_Frequency();
 
   /* Redraw radiation pattern on screen */
   if( isFlagSet(DRAW_ENABLED) )
@@ -1164,6 +1233,255 @@ Alloc_Nearfield_Buffers( int n1, int n2, int n3 )
 
 /*-----------------------------------------------------------------------*/
 
+/* Inverse_Scale_Gain()
+ *
+ * Calculates the actual dB value from a scaled gain value
+ * This is the inverse of Scale_Gain()
+ */
+static double
+Inverse_Scale_Gain(double scaled_val)
+{
+  double db_val = 0.0;
+
+  switch(rc_config.gain_style)
+  {
+    case GS_LINP:
+      if (scaled_val > 0.0)
+        db_val = 10.0 * log10(scaled_val);
+      else
+        db_val = -999.99;
+      break;
+
+    case GS_LINV:
+      if (scaled_val > 0.0)
+        db_val = 20.0 * log10(scaled_val);
+      else
+        db_val = -999.99;
+      break;
+
+    case GS_ARRL:
+      if (scaled_val > 0.0)
+        db_val = log(scaled_val) / 0.058267;
+      else
+        db_val = -999.99;
+      break;
+
+    case GS_LOG:
+      db_val = (scaled_val - 1.0) * 40.0;
+      break;
+  }
+
+  return db_val;
+}
+
+/* Draw_Color_Legend_Overlay()
+ *
+ * Draws a color gradient legend in the bottom right corner
+ * showing gain values and corresponding colors
+ */
+void
+Draw_Color_Legend_Overlay( cairo_t *cr )
+{
+  /* Declare all variables at start */
+  int i, x, y;
+  double red, grn, blu;
+  char txt[16];
+  int fstep = calc_data.freq_step;
+  int pol = calc_data.pol_type;
+
+  double max_gain, min_gain, color_min;
+  double grad_pos, pos, db_val;
+  const int width = 20;
+  const int height = 200;
+  const int margin = 10;
+  const int text_width = 80; /* Increased for "dB" suffix */
+  const int num_graduations = 5; /* Number of intermediate graduations */
+
+  /* Validate parameters */
+  if (!cr || !rad_pattern) return;
+
+
+  /* Validate frequency step. Note fstep == calc_data.steps_total is valid
+   * because an extra index is used the custom frequency when clicking on 
+   * the graph at an arbitrary point.
+   */
+  if (fstep < 0 || fstep > calc_data.steps_total) {
+    pr_err("fstep=%d is out of bounds. calc_data.steps_total=%d", fstep, calc_data.steps_total);
+    return;
+  }
+
+  /* Validate polarization type */
+  if (pol < 0 || pol >= NUM_POL) return;
+
+  /* Validate rad_pattern members */
+  if (!rad_pattern[fstep].max_gain || !rad_pattern[fstep].min_gain) return;
+  
+  /* Position in bottom right corner */
+  x = rdpattern_proj_params.width - width - text_width - margin;
+  y = rdpattern_proj_params.height - height - margin;
+
+  /* Get actual min/max gains and scale them */
+  max_gain = rad_pattern[fstep].max_gain[pol];
+  min_gain = rad_pattern[fstep].min_gain[pol];
+  color_min = (min_gain < COLOR_MIN_GAIN) ? COLOR_MIN_GAIN : min_gain;
+
+  /* Scale the gains for color mapping */
+  double scaled_max = Scale_Gain(max_gain, fstep, rad_pattern[fstep].max_gain_idx[pol]);
+  double scaled_min = Scale_Gain(color_min, fstep, rad_pattern[fstep].min_gain_idx[pol]);
+  double scaled_range = scaled_max - scaled_min;
+
+  /* Calculate relative gain points starting at -3dB and stepping by -6dB */
+  int num_rel_marks = 0;
+  double rel_gains[10]; // Large enough for all possible marks
+  double scaled_vals[10];
+  double positions[10];
+  
+  // Get text height for spacing calculation
+  cairo_text_extents_t text_extents;
+  cairo_text_extents(cr, "-88 dB", &text_extents);
+  double text_height = text_extents.height;
+  double min_mark_spacing = text_height * 1.5; // Minimum pixels between marks
+  
+  // Start with -3dB
+  rel_gains[num_rel_marks++] = -3.0;
+  
+  // Add -6dB steps and -10dB multiples
+  double current_gain = -6.0;
+  double last_y_pos = 0;
+  
+  while (current_gain > COLOR_MIN_GAIN) { // Reasonable lower limit
+    double gain_val = max_gain + current_gain;
+    double scaled_val = Scale_Gain(gain_val, fstep, rad_pattern[fstep].max_gain_idx[pol]);
+    double pos = (scaled_val - scaled_min) / scaled_range;
+    double y_pos = (1.0 - pos) * (height - 1);
+    
+    // Only add mark if it has enough spacing from previous mark
+    if (num_rel_marks == 1 || (y_pos - last_y_pos) >= min_mark_spacing) {
+      rel_gains[num_rel_marks++] = current_gain;
+      last_y_pos = y_pos;
+    }
+    current_gain -= 6.0;
+  }
+
+  // Calculate scaled values and positions
+  for (i = 0; i < num_rel_marks; i++) {
+    double gain_val = max_gain + rel_gains[i];
+    scaled_vals[i] = Scale_Gain(gain_val, fstep, rad_pattern[fstep].max_gain_idx[pol]);
+    positions[i] = (scaled_vals[i] - scaled_min) / scaled_range;
+  }
+
+  /* Draw color gradient bar */
+  if (x >= 0 && y >= 0 &&
+      x + width <= rdpattern_proj_params.width &&
+      y + height <= rdpattern_proj_params.height)
+  {
+    /* Draw headings */
+    cairo_set_source_rgb(cr, WHITE);
+    cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+    
+    /* Right-justify "Rel." */
+    cairo_text_extents_t rel_extents;
+    cairo_text_extents(cr, "Rel.", &rel_extents);
+    cairo_move_to(cr, x - 5 - rel_extents.width, y - 7);  /* Moved up 2px */
+    cairo_show_text(cr, "Rel.");
+    
+    /* Left-justify "Abs." */
+    cairo_move_to(cr, x + width + 5, y - 7);  /* Moved up 2px */
+    cairo_show_text(cr, "Abs.");
+
+    /* Draw color gradient with proper scaling (min at bottom, max at top) */
+    for (i = 0; i < height; i++)
+    {
+      /* Map position in gradient (i=0 is top, i=height-1 is bottom) */
+      pos = 1.0 - ((double)i / (double)(height - 1));  // Invert position
+      
+      /* Get scaled value at this position */
+      double scaled_val = scaled_min + (pos * scaled_range);
+      
+      /* Convert to color */
+      Value_to_Color(&red, &grn, &blu, (scaled_val - scaled_min) / scaled_range, 1.0);
+      cairo_set_source_rgb(cr, red, grn, blu);
+      cairo_rectangle(cr, x, y + i, width, 1);
+      cairo_fill(cr);
+    }
+
+    /* Draw border around gradient */
+    cairo_set_source_rgb(cr, WHITE);
+    cairo_set_line_width(cr, LINE_WIDTH);
+    cairo_rectangle(cr, x, y-(LINE_WIDTH/2), width, height+2*(LINE_WIDTH/2));
+    cairo_stroke(cr);
+
+    /* Draw graduation marks with matching colors and values (min at bottom, max at top) */
+    for (i = 0; i < num_graduations; i++)
+    {
+      /* Calculate scaled value at this graduation (reverse order) */
+      grad_pos = (double)(num_graduations - 1 - i) / (double)(num_graduations - 1);
+      
+      /* Get scaled value at this position */
+      double scaled_val = scaled_min + (grad_pos * scaled_range);
+      
+      /* Convert back to dB for display */
+      db_val = Inverse_Scale_Gain(scaled_val);
+      
+      /* Draw graduation mark with matching color */
+      Value_to_Color(&red, &grn, &blu, grad_pos, 1.0);
+      cairo_set_source_rgb(cr, red, grn, blu);
+      
+      int grad_y = y + (i * height) / (num_graduations - 1);
+      cairo_move_to(cr, x + width - (LINE_WIDTH/2), grad_y);
+      cairo_line_to(cr, x + width + TEXT_GRADIENT_SPACING, grad_y);
+      cairo_stroke(cr);
+
+      /* Show gain value in text */
+      if (db_val < -999.99) db_val = -999.99;
+      /* First measure width of numeric part */
+      char num_txt[16];
+      snprintf(num_txt, sizeof(num_txt)-1, "%.2f", db_val);
+      cairo_text_extents_t num_extents;
+      cairo_text_extents(cr, num_txt, &num_extents);
+      
+      /* Then format full text with dBi */
+      snprintf(txt, sizeof(txt)-1, "%.2f dBi", db_val);
+      cairo_set_source_rgb(cr, WHITE);
+      /* Right-justify absolute gain values with normal weight */
+      cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
+      
+      /* Position text so numeric part aligns with gradient */
+      cairo_move_to(cr, x + width + TEXT_GRADIENT_SPACING, grad_y + 4);
+      cairo_show_text(cr, txt);
+    }
+
+    /* Draw relative gain marks on left side */
+    for (i = 0; i < num_rel_marks; i++) {
+      int mark_y = y + (int)((1.0 - positions[i]) * (height - 1));
+      
+      /* Draw mark */
+      Value_to_Color(&red, &grn, &blu, positions[i], 1.0);
+      cairo_set_source_rgb(cr, red, grn, blu);
+      cairo_move_to(cr, x - TEXT_GRADIENT_SPACING, mark_y);
+      cairo_line_to(cr, x, mark_y);
+      cairo_stroke(cr);
+
+      /* Show value */
+      cairo_set_source_rgb(cr, WHITE);
+      if (i == 0) {  /* -3dB point */
+        /* Ensure -3dB is bold */
+        cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+        snprintf(txt, sizeof(txt)-1, "-3 dB");
+      } else {
+        /* Other relative gain values in normal weight */
+        cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
+        snprintf(txt, sizeof(txt)-1, "%d dB", (int)rel_gains[i]);
+      }
+      /* Right-justify relative gain values */
+      cairo_text_extents_t extents;
+      cairo_text_extents(cr, txt, &extents);
+      cairo_move_to(cr, x - TEXT_GRADIENT_SPACING - extents.width, mark_y + 4);
+      cairo_show_text(cr, txt);
+    }
+  }
+}
+
 /* Free_Draw_Buffers()
  *
  * Frees the buffers used for drawing
