@@ -83,7 +83,7 @@ int write_exact(int fd, char *buf, int size)
 		if (len < 0 && errno == EINTR) continue;
 
 		if (len < 0) {
-			perror("write_exact(): ");
+			perror("write()");
 			return len;
 		}
 
@@ -110,7 +110,7 @@ int read_exact(int fd, char *buf, int size)
 		if (len < 0 && errno == EINTR) continue;
 
 		if (len < 0) {
-			perror("read_exact(): ");
+			perror("read()");
 			return len;
 		}
 
@@ -146,7 +146,7 @@ Read_Pipe( int idx, char *str, ssize_t len, gboolean err )
 
 	  if (retval == -1 && errno != EINTR)
 	  {
-		perror( "xnec2c: Read_Pipe(): select()" );
+		perror( "select()" );
 		_exit( 0 );
 	  }
   } while (retval == -1 && errno == EINTR);
@@ -155,9 +155,8 @@ Read_Pipe( int idx, char *str, ssize_t len, gboolean err )
 
   if( (retval == -1) || ((retval != len) && err ) )
   {
-    perror( "xnec2c: Read_Pipe(): read()" );
-    fprintf( stderr, "xnec2c: Read_Pipe(): child %d  length %d  return %d\n",
-        idx, (int)len, (int)retval );
+    perror( "read()" );
+    pr_err("child %d  length %d  return %d\n", idx, (int)len, (int)retval);
     _exit( 0 );
   }
   return( retval );
@@ -409,6 +408,9 @@ Child_Process( int num_child )
   FD_SET( forked_proc_data[num_child]->child2pnt_pipe[WRITE],
       &forked_proc_data[num_child]->write_fds );
 
+  // Scale the OpenMP resources based on the number of parallel forked jobs.
+  xnec2c_set_omp_cpus();
+
   /* Loop around select() in Read_Pipe() waiting for commands/data */
   while( TRUE )
   {
@@ -418,22 +420,22 @@ Child_Process( int num_child )
     switch( Fork_Command(cmnd) )
     {
       case MATHLIB:
-        retval = Read_Pipe( num_child,
-			(char*)&rc_config.mathlib_batch_idx,
-			sizeof(rc_config.mathlib_batch_idx), FALSE );
+        Read_Pipe( num_child,
+			rc_config.mathlib_batch_id,
+			MATHLIB_ID_LEN, FALSE );
 
         // Clear the previous frequency cache to prevent false values from benchmarking:
-        if (current_mathlib->idx != rc_config.mathlib_batch_idx)
+        if (strcmp(current_mathlib->id, rc_config.mathlib_batch_id) != 0)
             New_Frequency_Reset_Prev();
 
         // This says "interactive" mathlib, but since we are forked it is running
         // as a batch from the parent.
-        set_mathlib_interactive(NULL, get_mathlib_by_idx(rc_config.mathlib_batch_idx));
+        set_mathlib_interactive(NULL, get_mathlib_by_id(rc_config.mathlib_batch_id));
         break;
 
       case INFILE: /* Read input file */
         retval = Read_Pipe( num_child, rc_config.input_file, sizeof(rc_config.input_file), FALSE );
-        rc_config.input_file[retval] = '\0';
+        rc_config.input_file[retval-1] = '\0';
         Child_Input_File();
         break;
 
@@ -511,7 +513,7 @@ Write_Pipe( int idx, char *str, ssize_t len, gboolean err )
 
 	  if (retval == -1 && errno != EINTR)
 	  {
-		perror( "xnec2c: Write_Pipe(): select()" );
+		perror( "select()" );
 		_exit( 0 );
 	  }
 
@@ -521,7 +523,7 @@ Write_Pipe( int idx, char *str, ssize_t len, gboolean err )
   retval = write_exact( pipefd, str, (size_t)len );
   if( (retval == -1) || ((retval != len) && err) )
   {
-    perror( "xnec2c: Write_Pipe(): write()" );
+    perror( "write()" );
     _exit( 0 );
   }
 
@@ -543,16 +545,14 @@ static ssize_t PRead_Pipe(int idx, char *str, ssize_t len, gboolean err)
 	retval = read_exact(forked_proc_data[idx]->child2pnt_pipe[READ], str, (size_t) len);
 	if (retval < 0)
 	{
-		perror("xnec2c: PRead_Pipe(): read()");
-		fprintf(stderr, "xnec2c: PRead_Pipe(): child %d  length %d  return %d\n",
-			idx, (int) len, (int) retval);
+		perror("read()");
+		pr_err("child %d  length %d  return %d\n", idx, (int)len, (int)retval);
 		_exit(0);
 	}
 
 	if (retval == 0 && len > 0)
 	{
-		fprintf(stderr, "xnec2c: PRead_Pipe(): early EOF?, child %d  length %d  return %d\n",
-			idx, (int) len, (int) retval);
+		pr_err("early EOF?, child %d  length %d  return %d\n", idx, (int)len, (int)retval);
 		return -1;
 	}
 
@@ -568,7 +568,7 @@ static ssize_t PRead_Pipe(int idx, char *str, ssize_t len, gboolean err)
  *
  * Be sure to hold the freq_data_lock mutex when calling this function.
  */
-  void
+  int
 Get_Freq_Data( int idx, int fstep )
 {
   char *buff = NULL, flag;
@@ -606,7 +606,7 @@ Get_Freq_Data( int idx, int fstep )
 
   /* Notification to read near field data */
   if (PRead_Pipe( idx, nfeh, 4, TRUE ) < 0)
-	  return;
+	  return 0;
 
   nfeh[4] = '\0';
 
@@ -638,7 +638,7 @@ Get_Freq_Data( int idx, int fstep )
   if (PRead_Pipe( idx, buff, (ssize_t)buff_size, TRUE ) < 0)
   {
 	  free_ptr((void **)&buff);
-	  return;
+	  return 0;
   }
 
   /* Get current and charge data */
@@ -748,6 +748,7 @@ Get_Freq_Data( int idx, int fstep )
 
   free_ptr( (void **)&buff );
 
+  return 1;
 } /* Get_Freq_Data() */
 
 /*------------------------------------------------------------------------*/
