@@ -17,12 +17,16 @@
  *    https://www.xnec2c.org/
  */
 
+#include "config.h"
+
+#ifdef HAVE_BACKTRACE
 #include <execinfo.h>
+#endif
 
 #include "utils.h"
 #include "shared.h"
 
-char **_get_backtrace();
+char **_get_backtrace(void);
 void _print_backtrace(char **strings);
 
 
@@ -36,14 +40,32 @@ void _print_backtrace(char **strings);
   void
 usage(void)
 {
-  fprintf( stderr,
-      _("Usage: xnec2c <input-file-name>\n"
-        "              [-i <input-file-name>]\n"
-        "              [-j <number of processors in SMP machine>]\n"
-        "                  -j 0 disables forking for debug.\n"
-        "              [-P: debug: disable pthreads for freqloop, use GTK mainloop instead.\n"
-        "              [-h: print this usage information and exit]\n"
-        "              [-v: print xnec2c version number and exit]\n") );
+  fprintf(stdout, "Usage: xnec2c [options] [<input-file-name>]\n"
+		"  -i|--input <input-file-name>\n"
+		"  -j|--jobs  <number of processors in SMP machine> (-j0 disables forking)\n"
+		"     --openblas-threads <n>  set OpenBLAS thread count (default: 1)\n"
+		"     --mkl-threads <n>       set Intel MKL thread count (default: 1)\n"
+		"     --omp-threads <n>       set OpenMP thread count (default: 1)\n"
+		"  -b|--batch:        enable batch mode, exit after the frequency loop runs\n"
+		"     --optimize:     Activate the optimizer immediately.\n"
+		"  -P|--no-pthreads:  disable pthreads and use the GTK loop for debugging\n"
+		"  -h|--help:         print usage information and exit\n"
+		"  -V|--version:      print xnec2c version number and exit\n"
+		"  -v|--verbose:      increase verbosity, can be specified multiple times\n"
+		"  -d|--debug:        enable debug output (-dd includes backtraces)\n"
+		"  -d|--quiet:        suppress debug/verbose output\n"
+		"\n"
+		"The following arguments write to an output file after the frequency loop\n"
+		"completes.  These are useful to combine with --batch; If you wish to specify\n"
+		"filenames to write without --batch mode then enable the File->Optimizer\n"
+		"Settings or the files you specify on the command line will not be written.\n"
+		"\n"
+		"  --write-csv             <filename>  - write CSV file of measurements\n"
+		"  --write-s1p             <filename>  - write S1P file of S-parameters\n"
+		"  --write-s2p-max-gain    <filename>  - write S2P file, port-2 is max-gain\n"
+		"  --write-s2p-viewer-gain <filename>  - write S2P file, port-2 is viewer-gain\n"
+		"  --write-rdpat           <filename>  - write CSV of the radiation pattern\n"
+		"  --write-currents        <filename>  - write CSV of currents and charges\n");
 
 } /* end of usage() */
 
@@ -52,12 +74,32 @@ usage(void)
 int Notice(char *title, char *message,  GtkButtonsType buttons)
 {
 	int response;
+	int locked = 0;
+
+	if (!g_mutex_trylock(&freq_data_lock))
+		locked = 1;
+	else
+		g_mutex_unlock(&freq_data_lock);
+
+	if (!g_mutex_trylock(&global_lock))
+		locked = 1;
+	else
+		g_mutex_unlock(&global_lock);
+
+	if (locked)
+	{
+		pr_err("\n=== Notice: %s ===\n%s\n\n", title, message);
+
+		return 0;
+	}
+
+	pr_notice("\n=== Notice: %s ===\n%s\n\n", title, message);
+
 	GtkWidget *notice = gtk_message_dialog_new(GTK_WINDOW(main_window),
 		GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
 		buttons,
 		"%s", title);
 
-	printf("\n=== Notice: %s ===\n%s\n\n", title, message);
 	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(notice), "%s", message);
 
 	response = gtk_dialog_run(GTK_DIALOG(notice));
@@ -75,22 +117,41 @@ Stop( char *mesg, int err )
 {
   GtkBuilder *builder;
 
+  int locked = 0;
+
+  if (!g_mutex_trylock(&freq_data_lock))
+    locked = 1;
+  else
+    g_mutex_unlock(&freq_data_lock);
+
+  if (!g_mutex_trylock(&global_lock))
+    locked = 1;
+  else
+    g_mutex_unlock(&global_lock);
+
+  pr_err("Stop: %s\n", mesg);
+
   /* For child processes */
   if( CHILD )
   {
-    fprintf( stderr, "%s\n", mesg );
     if( err )
     {
-      fprintf( stderr,
-          _("xnec2c: fatal: child process %d exiting\n"), num_child_procs );
-      _exit(-1);
+      pr_crit("%s\n", mesg);
     }
-    else return( err );
+    else
+    {
+      pr_err("%s\n", mesg);
+    }
+    return( err );
 
   } /* if( CHILD ) */
 
-  /* Stop operation */
-  Stop_Frequency_Loop();
+  SetFlag(FREQ_LOOP_STOP);
+
+  if (!locked)
+    /* Stop operation */
+    Stop_Frequency_Loop();
+
 
   /* Create error dialog */
   if( !error_dialog )
@@ -112,7 +173,7 @@ Stop( char *mesg, int err )
   while( isFlagSet(ERROR_CONDX) )
   {
 	// We are out of gtk_main() so iterate manually:
-	while( g_main_context_iteration(NULL, FALSE) );
+	while( g_main_context_iteration(NULL, FALSE) ) {}
 
     if( isFlagSet(MAIN_QUIT) ) exit(-1);
 
@@ -146,7 +207,7 @@ Nec2_Save_Warn( const gchar *mesg )
     while( isFlagSet(ERROR_CONDX) )
     {
       // We are out of gtk_main() so iterate manually:
-      while( g_main_context_iteration(NULL, FALSE) );
+      while( g_main_context_iteration(NULL, FALSE) ) {}
 
       if( isFlagSet(MAIN_QUIT) ) exit(-1);
 
@@ -230,9 +291,63 @@ Load_Line( char *buff, FILE *pfile )
 
 /*------------------------------------------------------------------------*/
 
+// Scale the OpenMP resources based on the number of parallel forked jobs.
+// If you fork 25 jobs and have 100 CPUs then OMP can run 4 CPUs per fork:
+void xnec2c_set_omp_cpus(void)
+{
+#ifdef HAVE_OPENMP
+	int cpus_per_job;
+	int n;
+
+	int env_num_procs = 0;
+
+	// Respect OMP_NUM_THREADS
+	if (getenv("OMP_NUM_THREADS") != NULL)
+		env_num_procs = atoi(getenv("OMP_NUM_THREADS"));
+
+	if (env_num_procs > 0)
+		n = env_num_procs;
+	else
+		n = omp_get_num_procs();
+
+	if (CHILD)
+	{
+		cpus_per_job = n / calc_data.num_jobs;
+
+		if (cpus_per_job <= 0)
+			cpus_per_job = 1;
+
+		omp_set_num_threads(cpus_per_job);
+		pr_info("using %d OpenMP threads per job (%d/%d CPUs)\n",
+			getpid(),
+			cpus_per_job, omp_get_num_threads(), omp_get_num_procs());
+	}
+	else
+		omp_set_num_threads(n);
+#endif
+}
+
+void clock_print_elapsed_when(char *msg, clockid_t clk_id, struct timespec *start, float min_sec)
+{
+	struct timespec end;
+	clock_gettime(clk_id, &end);
+	double elapsed = (end.tv_sec + (double)end.tv_nsec/1e9) - (start->tv_sec + (double)start->tv_nsec/1e9);
+
+	if (elapsed > min_sec)
+		pr_debug("%s[%d]: %f seconds\n",
+			msg,
+			getpid(),
+			(end.tv_sec + (double)end.tv_nsec/1e9) - (start->tv_sec + (double)start->tv_nsec/1e9));
+}
+
+void clock_print_elapsed(char *msg, clockid_t clk_id, struct timespec *start)
+{
+	clock_print_elapsed_when(msg, clk_id, start, 0);
+}
+
 /* xnec2c uses mem_realloc() very often to resize buffers in the code.
  * There are cases where uninitialized memory buffers can lead to incorrect behavior
- * but unfortunatly the libc realloc() call doesn't initialize the reallocated memory.
+ * but unfortunately the libc realloc() call doesn't initialize the reallocated memory.
  * 
  * Since there is not a portable way to discover the amount of memory allocated by the
  * previous realloc/malloc() call we cannot call memset() on the extended portion.
@@ -266,6 +381,19 @@ typedef struct mem_obj_t
 	void *ptr;
 } mem_obj_t;
 
+void mem_obj_dump(void *ptr)
+{
+	mem_obj_t *m = (mem_obj_t*)ptr;
+	m--;
+
+	pr_debug("mem_obj_t at %p:\n"
+		"  size: %lu\n"
+		"  used: %lu\n"
+		"  addr: %p\n", (void *)m, (unsigned long)m->size,  (unsigned long)m->used, m->ptr);
+
+
+}
+
 void mem_backtrace(void *ptr)
 {
 	mem_obj_t *m = (mem_obj_t *)ptr;
@@ -277,9 +405,9 @@ void mem_backtrace(void *ptr)
 	}
 
 	m--;
-	printf("mem_backtrace(%p):\n", ptr);
+	pr_debug("mem_backtrace(%p):\n", ptr);
 	if (m->backtrace == NULL)
-		printf("  m->backtrace is null, no backtrace data\n");
+		pr_debug("  m->backtrace is null, no backtrace data\n");
 	else
 		_print_backtrace(m->backtrace);
 
@@ -344,7 +472,7 @@ void mem_realloc( void **ptr, size_t req, gchar *str )
   {
     snprintf( mesg, sizeof(mesg),
         _("Memory re-allocation denied %s\n"), str );
-    fprintf( stderr, "%s: Memory requested %lu\n", mesg, req );
+    pr_err("%s: Memory requested %lu\n", mesg, (unsigned long)req);
     Stop( mesg, ERR_STOP );
 	return;
   }
@@ -394,6 +522,8 @@ Open_File( FILE **fp, char *fname, const char *mode )
     Stop( mesg, ERR_STOP );
     return( FALSE );
   }
+  else
+    pr_debug("%s: %s\n", mode, fname);
 
   return( TRUE );
 } /* Open_File() */
@@ -592,6 +722,24 @@ double Strtod( char *nptr, char **endptr )
 
 /*------------------------------------------------------------------*/
 
+/* str_append()
+ *
+ * Append a and b, store in dst.
+ *
+ * For example, this writes "hello world" to d:
+ *    char d[20];
+ *    str_append(d, "hello ", "world", 19);
+ */
+char *str_append(char *dst, char *a, char *b, size_t n)
+{
+  Strlcpy( dst, a, n );
+  Strlcat( dst, b, n );
+
+  return dst;
+}
+
+/*------------------------------------------------------------------*/
+
 /* Get_Dirname()
  *
  * Gets the directory name from a file path
@@ -618,11 +766,14 @@ Get_Dirname( char *fpath, char *dirname, int *fname_idx )
 
 typedef struct 
 {
-	GSourceFunc function;
+	GSourceOnceFunc function;
 	gpointer data;
 	GMutex lock;
+	GCond cond;
 	int is_locked;
 	char **backtrace;
+
+	volatile int done;
 } g_idle_add_data_t;
 
 
@@ -638,7 +789,7 @@ int _callback_g_idle_add_once(g_idle_add_data_t *cbdata)
 	if (cbdata->is_locked)
 	{
 		// Wait for GTK
-		while (g_main_context_iteration(NULL, FALSE));
+		while (g_main_context_iteration(NULL, FALSE)) {}
 		
 		if (cbdata->backtrace != NULL)
 		{
@@ -646,6 +797,10 @@ int _callback_g_idle_add_once(g_idle_add_data_t *cbdata)
 			cbdata->backtrace = NULL;
 		}
 
+		// Signal the waiting thread:
+		g_mutex_lock(&cbdata->lock);
+		cbdata->done = 1;
+		g_cond_signal(&cbdata->cond);
 		g_mutex_unlock(&cbdata->lock);
 	}
 	else
@@ -663,46 +818,66 @@ int _callback_g_idle_add_once(g_idle_add_data_t *cbdata)
 	return FALSE;
 }
 
-void _g_idle_add_once(GSourceFunc function, gpointer data, int lock)
+guint _g_idle_add_once(GSourceOnceFunc function, gpointer data, int lock)
 {
+	guint ret;
 	g_idle_add_data_t *cbdata = NULL;
+
 	mem_alloc((void**)&cbdata, sizeof(g_idle_add_data_t), __LOCATION__);
 
-	g_mutex_init(&cbdata->lock);
+	// Don't do a locked sync if -P is specified because GTK
+	// is already running synchronously.
+	if (rc_config.disable_pthread_freqloop)
+		lock = 0;
+
+	if (lock)
+	{
+		g_mutex_init(&cbdata->lock);
+		g_cond_init(&cbdata->cond);
+	}
 
 	cbdata->function = function;
 	cbdata->data = data;
 	cbdata->is_locked = lock;
+	cbdata->done = 0;
 
 	cbdata->backtrace = NULL;
 
 	// Debug async call backtraces if you need it, but be aware that _get_backtrace() is slow.
 	//cbdata->backtrace = _get_backtrace();
 
-	if (lock)
-		g_mutex_lock(&cbdata->lock);
-
-	g_idle_add((GSourceFunc)_callback_g_idle_add_once, cbdata);
+	ret = g_idle_add((GSourceFunc)_callback_g_idle_add_once, cbdata);
 
 	// Wait for the lock to release and free it.
 	if (lock)
 	{
 		g_mutex_lock(&cbdata->lock);
+		while (!cbdata->done)
+			g_cond_wait(&cbdata->cond, &cbdata->lock);
 		g_mutex_unlock(&cbdata->lock);
+
+		g_mutex_clear(&cbdata->lock);
+		g_cond_clear(&cbdata->cond);
+
 		free_ptr((void**)&cbdata);
 	}
+
+	return ret;
 }
 
 // Call from any thread to queue a function to run once, do not wait for it to finish.
-void g_idle_add_once(GSourceFunc function, gpointer data)
+// This was implemented in glib 2.74 so use Gnome's version if available.
+#if GLIB_VERSION_CUR_STABLE < G_ENCODE_VERSION(2,74)
+guint g_idle_add_once(GSourceOnceFunc function, gpointer data)
 {
-	_g_idle_add_once(function, data, 0); // async
+	return _g_idle_add_once(function, data, 0); // async
 }
+#endif
 
 // Call from another thread to queue a function to run once, and wait for it to finish.
-void g_idle_add_once_sync(GSourceFunc function, gpointer data)
+guint g_idle_add_once_sync(GSourceOnceFunc function, gpointer data)
 {
-	_g_idle_add_once(function, data, 1); // sync
+	return _g_idle_add_once(function, data, 1); // sync
 }
 
 
@@ -711,12 +886,22 @@ void g_idle_add_once_sync(GSourceFunc function, gpointer data)
 void xnec2_widget_queue_draw(GtkWidget *w)
 {
 	// Only redraw the rdpattern when FREQ_LOOP_DONE or it the window will flash grey:
-	if (w == rdpattern_drawingarea && isFlagSet(OPTIMIZER_OUTPUT) && isFlagSet(FREQ_LOOP_RUNNING))
+	if (w == rdpattern_drawingarea &&
+	    isFlagSet(OPTIMIZER_OUTPUT) &&
+	    isFlagSet(FREQ_LOOP_RUNNING) &&
+	    !need_rdpat_redraw)
 	{
-		printf("Optimizer loop incomplete, skipping radiation pattern redraw.\n");
+		pr_debug("Optimizer loop incomplete, skipping radiation pattern redraw.\n");
+	}
+	else if (w == structure_drawingarea &&
+	    isFlagSet(OPTIMIZER_OUTPUT) &&
+	    isFlagSet(FREQ_LOOP_RUNNING) &&
+	    !need_structure_redraw)
+	{
+		pr_debug("Optimizer loop incomplete, skipping structure redraw.\n");
 	}
 	else
-		g_idle_add_once((GSourceFunc)gtk_widget_queue_draw, w);
+		g_idle_add_once((GSourceOnceFunc)gtk_widget_queue_draw, w);
 
 }
 /*------------------------------------------------------------------*/
@@ -728,14 +913,15 @@ void xnec2_widget_queue_draw(GtkWidget *w)
 void _print_backtrace(char **strings)
 {
 	int i;
-	printf("  Backtrace:\n");
+	pr_debug("  Backtrace:\n");
 	for (i = 0; strings[i] != NULL; i++)
-		printf("    %d. %s\n", i, strings[i]);
+		pr_debug("    %d. %s\n", i, strings[i]);
 }
 
 // Return an array of backtrace strings.  The value returned must be free()'ed.
-char **_get_backtrace()
+char **_get_backtrace(void)
 {
+#ifdef HAVE_BACKTRACE
 	void *array[10];
 	char **strings;
 	int size;
@@ -748,12 +934,15 @@ char **_get_backtrace()
 	strings[size-1] = NULL;
 
 	return strings;
+#else
+	return NULL;
+#endif
 }
 
 void print_backtrace(char *msg)
 {
 	if (msg != NULL)
-		printf("%s:\n", msg);
+		pr_debug("%s:\n", msg);
 
 	char **strings = _get_backtrace();
 	if (strings != NULL)
