@@ -30,12 +30,62 @@ static double tmp1, tmp2, tmp3, tmp4, tmp5, tmp6;
 
 /*-----------------------------------------------------------------------*/
 
+/* Set the calc_data.freq_step if it matches calc_data.freq_mhz.
+ * Redo radiation pattern for a new frequency.
+ *
+ * If it doesn't, return 0 so the caller can run New_Frequency() and
+ * use the extra buffer (in rad_pattern and other structures). */
+int set_freq_step(void)
+{
+	int fr, step;
+	double freq;
+
+	int prev_freq_step = calc_data.freq_step;
+
+	int idx = 0;
+	int found = 0;
+	for (fr = 0; !found && fr < calc_data.FR_cards && save.fstep[idx]; fr++)
+	{
+		freq = calc_data.freq_loop_data[fr].min_freq;
+		for (step = 0; !found && step < calc_data.freq_loop_data[fr].freq_steps && save.fstep[idx]; step++)
+		{
+			// if calc_data.freq_mhz =~ freq, +/- 1 Hz for rounding error:
+			if (calc_data.freq_mhz > freq - 1e-6 && calc_data.freq_mhz < freq + 1e-6)
+			{
+				calc_data.freq_step = idx;
+				found = 1;
+			}
+			else
+			{
+				if (calc_data.freq_loop_data[fr].ifreq == 1)
+					freq *= calc_data.freq_loop_data[fr].delta_freq;
+				else
+					freq += calc_data.freq_loop_data[fr].delta_freq;
+
+				idx++;
+			}
+
+		}
+	}
+
+	// If we didn't find the frequency, then use the "extra" frequency
+	// allocated as +1 at the end of all per-frequency data indexes:
+	if (!found)
+		calc_data.freq_step = calc_data.steps_total;
+
+	if (calc_data.freq_step != prev_freq_step)
+		SetFlag( DRAW_NEW_RDPAT );
+
+	// If we found the index, then no need to re-run New_Frequency because it is
+	// in the index.
+	return found;
+}
 /* Frequency_Scale_Geometry()
  *
  * Scales geometric parameters to frequency
  */
   static void
-Frequency_Scale_Geometry()
+Frequency_Scale_Geometry(void)
 {
   double fr;
   int idx;
@@ -132,12 +182,9 @@ Ground_Parameters( void )
         gnd.frati =( epsc - 1.0) / ( epsc + 1.0);
         if( cabs(( ggrid.epscf - epsc) / epsc) >= 1.0e-3 )
         {
-          fprintf( stderr,
-              _("xnec2c: Ground_Parameters(): error in ground parameters\n"
-              "complex dielectric constant from file: %12.5E%+12.5Ej\n"
-              "                            requested: %12.5E%+12.5Ej\n"),
-              creal(ggrid.epscf), cimag(ggrid.epscf),
-              creal(epsc), cimag(epsc) );
+          pr_err("complex dielectric constant from file: %12.5E%+12.5Ej, requested: %12.5E%+12.5Ej\n",
+                 creal(ggrid.epscf), cimag(ggrid.epscf),
+				 creal(epsc), cimag(epsc));
           Stop( _("Ground_Parameters():"
                 "Error in ground parameters"), ERR_STOP );
         }
@@ -268,12 +315,15 @@ Set_Network_Data( void )
   netcx.ntsol = 1;
 
   /* Save impedance data for normalization */
+  int fstep = calc_data.freq_step;
+  if (fstep < 0 || fstep > calc_data.steps_total)
+	return;
+
   if( ((calc_data.steps_total > 1) &&
-        isFlagSet(FREQ_LOOP_RUNNING)) || CHILD )
+        isFlagSet(FREQ_LOOP_RUNNING)) ||
+		CHILD ||
+		fstep == calc_data.steps_total)
   {
-    int fstep = calc_data.freq_step;
-    if (fstep < 0)
-        return;
 
     impedance_data.zreal[fstep] = (double)creal( netcx.zped);
     impedance_data.zimag[fstep] = (double)cimag( netcx.zped);
@@ -400,8 +450,7 @@ New_Frequency( void )
   save.last_freq = calc_data.freq_mhz;
 
   // Only show this if you manually change frequencies:
-  if (isFlagClear( FREQ_LOOP_RUNNING ))
-	clock_gettime(CLOCK_MONOTONIC, &start);
+  clock_gettime(CLOCK_MONOTONIC, &start);
 
   /* Frequency scaling of geometric parameters */
   Frequency_Scale_Geometry();
@@ -434,22 +483,12 @@ New_Frequency( void )
 
   g_mutex_unlock(&freq_data_lock);
 
-
-
-  // Only show this if you manually change frequencies:
-  if (isFlagClear( FREQ_LOOP_RUNNING ))
-  {
-	// Calculate elapsed time
-	clock_gettime(CLOCK_MONOTONIC, &end);
-
-	elapsed = (end.tv_sec + (double)end.tv_nsec/1e9) - (start.tv_sec + (double)start.tv_nsec/1e9);
-	printf("New_Frequency[%d]: %s: total time at %.2f MHz: %f seconds\n",
-		  getpid(),
-		  current_mathlib->name,
-		  calc_data.freq_mhz,
-		  elapsed
-	  );
-  }
+  // Calculate elapsed time
+  clock_gettime(CLOCK_MONOTONIC, &end);
+  
+  elapsed = (end.tv_sec + (double)end.tv_nsec/1e9) - (start.tv_sec + (double)start.tv_nsec/1e9);
+  pr_info("%.6f MHz: %f seconds. (%s)\n",
+			calc_data.freq_mhz, elapsed, current_mathlib->name);
 
 } /* New_Frequency()  */
 
@@ -460,8 +499,8 @@ static gboolean retval; /* Function's return value */
 int update_freqplots_fmhz_entry(gpointer p)
 {
     /* Display current frequency in plots entry */
-    char txt[10];
-    snprintf( txt, sizeof(txt), "%9.3f", calc_data.freq_mhz );
+    char txt[16];
+    snprintf( txt, sizeof(txt)-1, "%.3f", calc_data.freq_mhz );
     gtk_entry_set_text( GTK_ENTRY(Builder_Get_Object(
             freqplots_window_builder, "freqplots_fmhz_entry")), txt );
 
@@ -592,7 +631,8 @@ gboolean Frequency_Loop( gpointer udata )
    * execute only once, since only one instance is running */
   for( idx = 0; idx < calc_data.num_jobs; idx++ )
   {
-    /* Up frequency step count */
+    /* Up frequency step count. Note that this is initialized at -1 above so it
+	 * really does start the first loop with fstep==0 */
     fstep++;
 
     /* Set `freq` for use below
@@ -664,10 +704,9 @@ gboolean Frequency_Loop( gpointer udata )
           num_busy_procs++;
 
           // Send the mathlib to use, try to lock it if it is Intel MKL.
-		  mathlib_lock_intel_batch(rc_config.mathlib_batch_idx);
+		  mathlib_lock_intel_batch(rc_config.mathlib_batch_id);
           Write_Pipe( idx, fork_commands[MATHLIB], (ssize_t)strlen(fork_commands[MATHLIB]), TRUE );
-          Write_Pipe( idx, (char*)&rc_config.mathlib_batch_idx,
-			  (ssize_t)sizeof(rc_config.mathlib_batch_idx), TRUE );
+          Write_Pipe( idx, rc_config.mathlib_batch_id, (ssize_t)MATHLIB_ID_LEN, TRUE );
 
           /* Tell process to calculate freq dependent data */
           len = strlen( fork_commands[FRQDATA] );
@@ -688,9 +727,14 @@ gboolean Frequency_Loop( gpointer udata )
       calc_data.freq_mhz  = freq;
       calc_data.freq_step = fstep;
       calc_data.last_step = fstep;
-      g_mutex_lock(&freq_data_lock);
+      g_mutex_unlock(&freq_data_lock);
 
       New_Frequency();
+
+      // Be sure to exit if this was the last iteration:
+      if (fstep >= calc_data.steps_total-1)
+             retval = 0;
+
       break;
     }
 
@@ -720,7 +764,7 @@ gboolean Frequency_Loop( gpointer udata )
       {
         if (errno == EINTR)
 			continue;
-        perror( "xnec2c: select()" );
+        perror( "select()" );
         _exit(0);
       }
 
@@ -732,7 +776,14 @@ gboolean Frequency_Loop( gpointer udata )
         if( FD_ISSET(forked_proc_data[idx]->child2pnt_pipe[READ], &read_fds) )
         {
           /* Read data from finished child process */
-          Get_Freq_Data( idx, forked_proc_data[idx]->fstep );
+          if (!Get_Freq_Data( idx, forked_proc_data[idx]->fstep ))
+          {
+            pr_err("Failed to read data from forked child\n");
+            SetFlag(FREQ_LOOP_STOP);
+            g_mutex_unlock(&freq_data_lock);
+            g_mutex_unlock(&global_lock);
+            return FALSE;
+          }
 
           /* Clear "last-used-frequency" buffer, the local version of the data is
            * no longer what New_Frequency() set it to: */
@@ -763,20 +814,17 @@ gboolean Frequency_Loop( gpointer udata )
 		  retval = FALSE;
 		  if (num_busy_procs != 0)
 		  {
-			  printf("BUG: Frequency_Loop is done with all steps but num_busy_procs=%d (calc_data.freq_step=%d calc_data.steps_total=%d)?\n",
-				num_busy_procs,
-				calc_data.freq_step,
-				calc_data.steps_total);
+			  pr_debug("Frequency_Loop is done with all steps but num_busy_procs=%d (calc_data.freq_step=%d calc_data.steps_total=%d)?\n",
+                              num_busy_procs, calc_data.freq_step,
+                              calc_data.steps_total);
 				for( job_num = 0; job_num < calc_data.num_jobs; job_num++ )
 				{
 					if (forked_proc_data[job_num]->busy)
 					{
-						printf("  busy job[%d]: fstep=%d freq=%f\n",
+						pr_debug("  busy job[%d]: fstep=%d freq=%f\n",
 							job_num,
 							forked_proc_data[job_num]->fstep,
 							save.freq[forked_proc_data[job_num]->fstep]);
-
-
 					}
 				}
 		  }
@@ -810,7 +858,7 @@ gboolean Frequency_Loop( gpointer udata )
 	if( isFlagSet(PLOT_ENABLED) )
 	{
 	  /* Display current frequency in plots entry */
-	  g_idle_add_once((GSourceFunc)update_freqplots_fmhz_entry, NULL);
+	  g_idle_add_once((GSourceOnceFunc)update_freqplots_fmhz_entry, NULL);
 
 	  if( isFlagClear(OPTIMIZER_OUTPUT) || freqplots_click_pending())
 	  {
@@ -819,11 +867,11 @@ gboolean Frequency_Loop( gpointer udata )
 	}
 
 	/* Set main window frequency spinbutton */
-	g_idle_add_once((GSourceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
+	g_idle_add_once((GSourceOnceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
 
 	/* Set Radiation pattern window frequency spinbutton */
 	if( isFlagSet(DRAW_ENABLED) )
-		  g_idle_add_once((GSourceFunc)update_freq_mhz_spin_button_value, rdpattern_frequency);
+		  g_idle_add_once((GSourceOnceFunc)update_freq_mhz_spin_button_value, rdpattern_frequency);
 
 	xnec2_widget_queue_draw( structure_drawingarea );
   }
@@ -835,8 +883,9 @@ gboolean Frequency_Loop( gpointer udata )
     SetFlag( FREQ_LOOP_DONE );
 
 	clock_gettime(CLOCK_MONOTONIC, &end);
-	printf("Frequency_Loop elapsed time: %f seconds\n", 
-		(end.tv_sec + (double)end.tv_nsec/1e9) - (start.tv_sec + (double)start.tv_nsec/1e9));
+	pr_notice("Frequency loop elapsed time: %f seconds. (%s)\n",
+                (end.tv_sec + (double)end.tv_nsec / 1e9) - (start.tv_sec + (double)start.tv_nsec / 1e9),
+				(FORKED ? get_mathlib_by_id(rc_config.mathlib_batch_id)->name : current_mathlib->name));
 
     /* After the loop is finished, re-set the saved frequency
      * that the user clicked on in the frequency plots window */
@@ -858,17 +907,17 @@ gboolean Frequency_Loop( gpointer udata )
 
       /* Set main window frequency spinbutton.
 	   * This will trigger New_Frequency() via Redo_Currents(). */
-      g_idle_add_once_sync((GSourceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
+      g_idle_add_once_sync((GSourceOnceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
 
       /* Set Radiation pattern window frequency spinbutton.
 	   * This will trigger New_Frequency() via Redo_Radiation_Pattern,. */
       if( isFlagSet(DRAW_ENABLED) )
-        g_idle_add_once_sync((GSourceFunc)update_fmhz_save_spin_button_value, rdpattern_frequency);
+        g_idle_add_once_sync((GSourceOnceFunc)update_fmhz_save_spin_button_value, rdpattern_frequency);
 
       if( isFlagSet(PLOT_ENABLED) )
       {
         SetFlag( PLOT_FREQ_LINE );
-        g_idle_add_once_sync((GSourceFunc)update_freqplots_fmhz_entry, NULL);
+        g_idle_add_once_sync((GSourceOnceFunc)update_freqplots_fmhz_entry, NULL);
       }
     }
 
@@ -878,11 +927,13 @@ gboolean Frequency_Loop( gpointer udata )
     if( isFlagSet(DRAW_ENABLED) )
       xnec2_widget_queue_draw( rdpattern_drawingarea );
 
-    /* Write out frequency loop data for
-     * the optimizer if SIGHUP received */
-    if( isFlagSet(OPTIMIZER_OUTPUT) )
+    /* Write out frequency loop data for the optimizer if optimization is
+     * active.  This is a sync call, so only call this if we have files flagged
+     * for writing (opt_have_files_to_save) based on the checkboxes in
+     * File->Optimization Settings */
+    if( (rc_config.batch_mode || isFlagSet(OPTIMIZER_OUTPUT)) && opt_have_files_to_save() )
     {
-      g_idle_add_once_sync((GSourceFunc)Write_Optimizer_Data, NULL);
+      g_idle_add_once_sync((GSourceOnceFunc)Write_Optimizer_Data, NULL);
     }
   } // if( !retval && !num_busy_procs )
 
@@ -896,9 +947,25 @@ gboolean Frequency_Loop( gpointer udata )
 
 void *Frequency_Loop_Thread(void *p)
 {
+	// Don't draw the green line if in batch mode
+	if (rc_config.batch_mode)
+		calc_data.fmhz_save = 0.0;
+
+	// Run the loop
 	while (isFlagSet(FREQ_LOOP_RUNNING) && Frequency_Loop(NULL));
 
 	ClearFlag(FREQ_LOOP_RUNNING);
+
+	if (isFlagSet(FREQ_LOOP_STOP))
+		return NULL;
+
+	// Exit if in batch mode
+	if (rc_config.batch_mode)
+	{
+		g_idle_add_once_sync((GSourceOnceFunc)Gtk_Quit, NULL);
+		return NULL;
+	}
+
 	SetFlag(DRAW_NEW_RDPAT);
 
 	/*
@@ -916,14 +983,18 @@ void *Frequency_Loop_Thread(void *p)
 	   Re-draw drawing areas at end of loop 
 	 */
 	if (isFlagSet(PLOT_ENABLED))
-		g_idle_add_once_sync((GSourceFunc) gtk_widget_queue_draw, freqplots_drawingarea);
+		g_idle_add_once_sync((GSourceOnceFunc) gtk_widget_queue_draw, freqplots_drawingarea);
 
 	if (isFlagSet(DRAW_ENABLED))
 	{
-		g_idle_add_once_sync((GSourceFunc) update_freq_mhz_spin_button_value, rdpattern_frequency);
-		g_idle_add_once_sync((GSourceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
-		g_idle_add_once_sync(Redo_Currents, NULL);
-		g_idle_add_once_sync((GSourceFunc) gtk_widget_queue_draw, rdpattern_drawingarea);
+		g_idle_add_once_sync((GSourceOnceFunc)update_freq_mhz_spin_button_value, rdpattern_frequency);
+		g_idle_add_once_sync((GSourceOnceFunc)update_freq_mhz_spin_button_value, mainwin_frequency);
+		g_idle_add_once_sync((GSourceOnceFunc)Redo_Currents, NULL);
+
+		need_rdpat_redraw = 1;
+		need_structure_redraw = 1;
+		g_idle_add_once_sync((GSourceOnceFunc)gtk_widget_queue_draw, structure_drawingarea);
+		g_idle_add_once_sync((GSourceOnceFunc)gtk_widget_queue_draw, rdpattern_drawingarea);
 	}
 
 	return NULL;
@@ -955,8 +1026,8 @@ Start_Frequency_Loop( void )
 		if( ret != 0 )
 		{
 			free_ptr((void**)&pth_freq_loop);
-			fprintf( stderr, "xnec2c: failed to start Frequency_Loop_Thread\n" );
-			perror( "xnec2c: pthread_create()" );
+			pr_crit("failed to start Frequency_Loop_Thread\n");
+			perror( "pthread_create()" );
 			exit( -1 );
 		}
 	}
@@ -981,6 +1052,7 @@ Stop_Frequency_Loop( void )
 {
   // Clearing this flag will cause the Frequency_Loop pthread to exit when it is done:
   ClearFlag( FREQ_LOOP_RUNNING );
+  SetFlag(FREQ_LOOP_STOP);
 
   if (!rc_config.disable_pthread_freqloop)
   {
@@ -990,6 +1062,13 @@ Stop_Frequency_Loop( void )
 		  pthread_join(*pth_freq_loop, NULL);
 		  free_ptr((void**)&pth_freq_loop);
 	  }
+
+	  ClearFlag(FREQ_LOOP_STOP);
+
+	  // Flush any pending GTK events. This is critical because any pending
+	  // events that may work upon GtkWidget's that change (or close) upon exit
+	  // from this function will fail.
+	  while( g_main_context_iteration(NULL, FALSE) ) {}
   }
   else if( floop_tag > 0 )
   {
